      subroutine app_qp(ig,sgn,u1,n1,p1,u2,n2,p2,u,n,p,g,mass,wdth)
c----------------------------------------------------------------------c
c Returns a vector array being a contraction of a triple pi+pi-photon 
c vertex of the scalar QED with the photon polarization vector and 
c a scalar
c
c Input: ig=0(fixed coupling g)/else(running coupling g),
c        sgn=+1, if scalars are writen in order pi-pi+,
c            -1, if scalars are writen in order pi+pi-,
c        u1 - vector array,
c        u2 - scalar array,
c        n1,n2,n - numbers of polarization states,
c        p1,p2 - momenta,
c        g - a triple pi+pi-photon coupling,
c        mass, wdth - mass and width of a particle represented by u2.
c Output: u - scalar array,
c         p - momentum of the scalar particle.
c All four momenta are cosidered as incoming to the vertex.
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 7 May, 2009; modified on 13 May, 2009;
c 19 March, 2015 (call to subroutine runcoupl implemented)
c 17 Feb. 2016 (call to subroutine runcoupl changed)
c 3 March, 2020 (quadruple precision version written)
c 23 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0), qp=kind(1.q0)
      integer :: ig,n1,n2,n,i1,i2,i,sgn

      real(dp) :: mass,wdth
      real(qp), dimension(0:3) :: p1,p2,p
      complex(dp) :: g,rg
      complex(qp) :: u1(0:3,n1),u2(1,n2),u(1,n),gden

      p=-p1-p2

      if (ig == 0) then
       rg=g
      else
c       call runcoupl(ig,p,rg)
       call runcoupl_qp(ig,-p1,rg) ! 17 Feb. 2016
      end if

      gden=rg/cmplx(p(0)**2-p(1)**2-p(2)**2-p(3)**2-mass**2,
     &                                                  mass*wdth,qp)

      i=0
      do i2=1,n2
       do i1=1,n1
         i=i+1
         u(1,i)=gden*u2(1,i2)*sgn*(u1(0,i1)*(p2(0)-p(0))
     &                            -u1(1,i1)*(p2(1)-p(1))
     &                            -u1(2,i1)*(p2(2)-p(2))
     &                            -u1(3,i1)*(p2(3)-p(3)))
       end do
      end do

      end subroutine app_qp
