      subroutine pvvmd_qp(ig,g,u1,p1,u2,p2,u,mat)
c----------------------------------------------------------------------c
c Returns a matrix element for a triple pi0-vector meson-vector meson 
c cotracted with a scalar u1 and vectors u2 and u
c
c Input: ig=0(fixed coupling g)/else(running coupling g),
c        g - a triple pi0-vector meson-vector meson coupling,
c        u1 - scalar,
c        u2,u - vectors,
c        p1,p2 - momenta.
c        
c Output: mat - the matrix element.
c All four momenta are cosidered as incoming to the vertex.
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 25 Sept., 2014; modified on 
c 19 March, 2015 (call to subroutine runcoupl implemented)
c 9 March, 2020 (quadruple precision version written)
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0), qp=kind(1.q0)
      integer :: ig

      real(qp), dimension(0:3) :: p1,p2,p
      complex(dp) :: g,rg
      complex(qp) :: u1(1),u2(0:3),u(0:3),mat

      p=-p1-p2

      if (ig == 0) then
       rg=g
      else
       call runcoupl_qp(ig,p,rg)
      end if

      mat=rg*u1(1)*(
     &      (-u2(1)*cmplx(p2(2)*p(3)-p2(3)*p(2),0.q0,qp)
     &       +u2(2)*cmplx(p2(1)*p(3)-p2(3)*p(1),0.q0,qp)
     &       -u2(3)*cmplx(p2(1)*p(2)-p2(2)*p(1),0.q0,qp))*u(0)
     &     -(-u2(0)*cmplx(p2(2)*p(3)-p2(3)*p(2),0.q0,qp)
     &       +u2(2)*cmplx(p2(0)*p(3)-p2(3)*p(0),0.q0,qp)
     &       -u2(3)*cmplx(p2(0)*p(2)-p2(2)*p(0),0.q0,qp))*u(1)
     &     -( u2(0)*cmplx(p2(1)*p(3)-p2(3)*p(1),0.q0,qp)
     &       -u2(1)*cmplx(p2(0)*p(3)-p2(3)*p(0),0.q0,qp)
     &       +u2(3)*cmplx(p2(0)*p(1)-p2(1)*p(0),0.q0,qp))*u(2)
     &     -(-u2(0)*cmplx(p2(1)*p(2)-p2(2)*p(1),0.q0,qp)
     &       +u2(1)*cmplx(p2(0)*p(2)-p2(2)*p(0),0.q0,qp)
     &       -u2(2)*cmplx(p2(0)*p(1)-p2(1)*p(0),0.q0,qp))*u(3))

      end subroutine pvvmd_qp
