      subroutine sff_qp(u1,n1,p1,u2,n2,p2,u,n,p,ip,g,mass,wdth)
c----------------------------------------------------------------------c
c Returns the spinor array
c                  u=u1*u2*(gp*P+ + gm*P-)*(p+m)/den
c
c Input: u1 - polarization vector,
c        u2 - spinor
c        p1,p2 - momenta,
c        n1,n2,n - numbers of polarization states,
c        g - a fermion-fermion-scalar boson coupling,
c        mass, wdth - a mass width of u
c        ip=+1/-1 for u representing a particle/an antiparticle
c All four momenta are cosidered as incoming to the vertex.
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 16 May, 2007 (name and interface changed, used to be vfkk); 
c                           modified on 
c 9 March, 2020 (quadruple precision version written)
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0), qp=kind(1.q0)
      integer :: n1,n2,n,ip,i1,i2,i
      real(qp), dimension(0:3) :: p1,p2,p
      real(dp) :: mass,wdth
      complex(dp) :: g(2),mf,mf2
      complex(qp) :: u1(1,n1),u2(4,n2),u(4,n),den,gp,gm,mgp,mgm,su2(4)
      complex(qp), dimension(4) :: pp,pm

      mf2=cmplx(mass**2,-mass*wdth,dp)
      mf=sqrt(mf2)

      p=p1+p2

      pp=ip*(/cmplx(p(0)-p(3),0.q0,qp),   ! (1,1)
     &        cmplx(-p(1),-p(2),qp),      ! (2,1)
     &        cmplx(-p(1),p(2),qp),       ! (1,2)
     &        cmplx(p(0)+p(3),0.q0,qp) /) ! (2,2)

      pm=(/pp(4),-pp(2),-pp(3),pp(1)/)

      den=pp(1)*pp(4)-pp(3)*pp(2)-mf2

      gp=g(1)/den
      gm=g(2)/den
      mgp=mf*gp
      mgm=mf*gm

      i=0

      do i2=1,n2
       su2=(/mgm*u2(1,i2)+gp*(pp(1)*u2(3,i2)+pp(3)*u2(4,i2)),
     &       mgm*u2(2,i2)+gp*(pp(2)*u2(3,i2)+pp(4)*u2(4,i2)),
     &       gm*(pm(1)*u2(1,i2)+pm(3)*u2(2,i2))+mgp*u2(3,i2),
     &       gm*(pm(2)*u2(1,i2)+pm(4)*u2(2,i2))+mgp*u2(4,i2)/)
       do i1=1,n1
        i=i+1
        u(:,i)=u1(1,i1)*su2
       end do
      end do
      
      end subroutine sff_qp
