      subroutine vpppkk(ig,ip,uv,nv,pv,u1,n1,p1,u2,n2,p2,u,n,p,g,
     &                                                       mass,wdth)
c----------------------------------------------------------------------c
c Returns a scalar array being a contraction of a quartic
c vector meson-pi0-pi+pi- coupling with a vector uv and scalars u1,u2
c
c Input: ig=0(fixed coupling g)/else(running coupling g),
c        ip = +1, if the output particle is pi+,
c        ip = -1, if the output particle is pi0 or pi-,
c        uv - vector array,
c        u1,u2 - scalar arrays,
c        nv,n1,n2,n - numbers of polarization states,
c        pv,p1,p2 - momenta,
c        g - a quartic vector meson-pi0-pi+pi- coupling,
c        mass, wdth - mass and width of a scalar particle.
c Output: u - scalar array,
c         p - its momentum.
c All four momenta are cosidered as incoming to the vertex.
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 15 Oct., 2014; modified on 29 Oct., 2014;
c 19 March, 2015 (call to subroutine runcoupl implemented)
c 6 April, 2015:
c an extra minus sign, resulting from missing i^2 if a quartic coupling 
c is present, introduced
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0)
      integer :: ig,ip,nv,n1,n2,n,iv,i1,i2,i

      real(dp) :: mass,wdth
      real(dp), dimension(0:3) :: pv,p1,p2,p
      complex(dp) :: uv(0:3,nv),u1(1,n1),u2(1,n2),u(1,n),g,rg,gden,
     &               r(0:3)

      p=-pv-p1-p2

      if (ig == 0) then
       rg=g
      else
       call runcoupl(ig,p,rg)
      end if

      gden=-ip*rg/cmplx(p(0)**2-p(1)**2-p(2)**2-p(3)**2-mass**2,
     &                                             mass*wdth,dp)

      r=(/cmplx(-(p1(1)*(p2(2)*p(3)-p2(3)*p(2))
     &           -p1(2)*(p2(1)*p(3)-p2(3)*p(1))
     &           +p1(3)*(p2(1)*p(2)-p2(2)*p(1))),0.d0,dp),
     &    cmplx( -p1(0)*(p2(2)*p(3)-p2(3)*p(2))
     &           +p1(2)*(p2(0)*p(3)-p2(3)*p(0))
     &           -p1(3)*(p2(0)*p(2)-p2(2)*p(0)),0.d0,dp),
     &    cmplx(  p1(0)*(p2(1)*p(3)-p2(3)*p(1))
     &           -p1(1)*(p2(0)*p(3)-p2(3)*p(0))
     &           +p1(3)*(p2(0)*p(1)-p2(1)*p(0)),0.d0,dp),
     &    cmplx( -p1(0)*(p2(1)*p(2)-p2(2)*p(1))
     &           +p1(1)*(p2(0)*p(2)-p2(2)*p(0))
     &           -p1(2)*(p2(0)*p(1)-p2(1)*p(0)),0.d0,dp)/)

      i=0
      do i2=1,n2
       do i1=1,n1
        do iv=1,nv
         i=i+1
         u(1,i)=gden*u1(1,i1)*u2(1,i2)*(uv(0,iv)*r(0)
     &         -uv(1,iv)*r(1)-uv(2,iv)*r(2)-uv(3,iv)*r(3))
        end do
       end do
      end do

      end subroutine vpppkk
