      subroutine vvvvkk(u1,n1,p1,l1,u2,n2,p2,l2,u3,n3,p3,l3,u,n,p,l4,
     &                                                      g,mass,wdth)
c----------------------------------------------------------------------c
c Returns a polarization vector array being a contraction of
c a quartic gauge boson vertex with three polarization vectors
c
c Input: u1,u2,u3 - vector arrays,
c        n1,n2,n3,n - numbers of polarization states,
c        p1,p2,p3 - momenta
c        l1,l2,l3,l4 - position of u1,u2,u3 and u in the vertex,
c        g - a quartic gauge boson coupling.
c Output: u - vector array,
c         p - its momentum.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 28 March, 2006; modified on 27 April, 2006, 
c                             20 May, 2007 (interface changed)
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0)
      integer :: n1,n2,n3,n,l1,l2,l3,l4,i1,i2,i3,i

      real(dp) :: mass,wdth
      real(dp), dimension(0:3) :: p1,p2,p3,p
      complex(dp) :: g,u1(0:3,n1),u2(0:3,n2),u3(0:3,n3),u(0:3,n),mv2,
     &               gden

      p=-p1-p2-p3

      mv2=cmplx(mass**2,-mass*wdth,dp)
      gden=g/(cmplx(p(0)**2-p(1)**2-p(2)**2-p(3)**2,0.d0,dp)-mv2)

      i=0

      if ((l1 < 3 .and. l2 < 3) .or. (l3 < 3 .and. l4 < 3)) then
       do i3=1,n3
        do i2=1,n2
         do i1=1,n1
          i=i+1
          u(:,i)=gden*(
     &             (u1(0,i1)*u3(0,i3)
     &             -u1(1,i1)*u3(1,i3)
     &             -u1(2,i1)*u3(2,i3)
     &             -u1(3,i1)*u3(3,i3))*u2(:,i2)
     &            +(u2(0,i2)*u3(0,i3)
     &             -u2(1,i2)*u3(1,i3)
     &             -u2(2,i2)*u3(2,i3)
     &             -u2(3,i2)*u3(3,i3))*u1(:,i1)
     &       -2.d0*(u1(0,i1)*u2(0,i2)
     &             -u1(1,i1)*u2(1,i2)
     &             -u1(2,i1)*u2(2,i2)
     &             -u1(3,i1)*u2(3,i2))*u3(:,i3))
          if (mass /= 0.d0) then
           u(:,i)=u(:,i)
     &              -(p(0)*u(0,i)-p(1)*u(1,i)
     &               -p(2)*u(2,i)-p(3)*u(3,i))*p/mv2
          end if
         end do
        end do
       end do
      else if ((l1 < 3 .and. l3 < 3) .or. (l2 < 3 .and. l4 < 3)) then
       do i3=1,n3
        do i2=1,n2
         do i1=1,n1
          i=i+1
          u(:,i)=gden*(
     &             (u1(0,i1)*u2(0,i2)
     &             -u1(1,i1)*u2(1,i2)
     &             -u1(2,i1)*u2(2,i2)
     &             -u1(3,i1)*u2(3,i2))*u3(:,i3)
     &            +(u3(0,i3)*u2(0,i2)
     &             -u3(1,i3)*u2(1,i2)
     &             -u3(2,i3)*u2(2,i2)
     &             -u3(3,i3)*u2(3,i2))*u1(:,i1)
     &       -2.d0*(u1(0,i1)*u3(0,i3)
     &             -u1(1,i1)*u3(1,i3)
     &             -u1(2,i1)*u3(2,i3)
     &             -u1(3,i1)*u3(3,i3))*u2(:,i2))
          if (mass /= 0.d0) then
           u(:,i)=u(:,i)
     &             -(p(0)*u(0,i)-p(1)*u(1,i)
     &              -p(2)*u(2,i)-p(3)*u(3,i))*p/mv2
          end if
         end do
        end do
       end do
      else if ((l1 < 3 .and. l4 < 3) .or. (l2 < 3 .and. l3 < 3)) then
       do i3=1,n3
        do i2=1,n2
         do i1=1,n1
          i=i+1
          u(:,i)=gden*(
     &             (u1(0,i1)*u3(0,i3)
     &             -u1(1,i1)*u3(1,i3)
     &             -u1(2,i1)*u3(2,i3)
     &             -u1(3,i1)*u3(3,i3))*u2(:,i2)
     &            +(u2(0,i2)*u1(0,i1)
     &             -u2(1,i2)*u1(1,i1)
     &             -u2(2,i2)*u1(2,i1)
     &             -u2(3,i2)*u1(3,i1))*u3(:,i3)
     &       -2.d0*(u3(0,i3)*u2(0,i2)
     &             -u3(1,i3)*u2(1,i2)
     &             -u3(2,i3)*u2(2,i2)
     &             -u3(3,i3)*u2(3,i2))*u1(:,i1))
          if (mass /= 0.d0) then
           u(:,i)=u(:,i)
     &             -(p(0)*u(0,i)-p(1)*u(1,i)
     &              -p(2)*u(2,i)-p(3)*u(3,i))*p/mv2
          end if
         end do
        end do
       end do
      end if

      end subroutine vvvvkk
