      subroutine couplsm(e,cmw,cmh)
c----------------------------------------------------------------------c
c Returns the standard model couplings
c 
c Input: e - electric charge,
c        cmw - complex W boson mass
c        cmh - complex Higgs boson mass
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 18 June, 2001; modified on 20 September, 2001; 23 June, 2005;
c                            28 March, 2006; 13 May, 2007
c----------------------------------------------------------------------c

      use inprms
      use couplsmkk
      use topan
      use vmesoncrit

      implicit none

      real(dp) :: e
      complex(dp) :: cmw,cmh

      complex(dp) :: sw,ccw2,cw,hff(2)

      sw=sqrt(csw2)
      ccw2=1.d0-csw2
      cw=sqrt(ccw2)

c Photon couplings to fermions:

      ann=ocom
      all=-cmplx(-e,0.d0,dp)
      auu=-cmplx(2.d0/3.d0*e,0.d0,dp)
      add=-cmplx(-1.d0/3.d0*e,0.d0,dp)

c Z boson couplings to fermions:

      znn=(/ocom,1.d0/(2.d0*sw*cw)*e/)
      zll=(/sw/cw*e,-(1.d0-2.d0*csw2)/(2.d0*sw*cw)*e/)
      zuu=(/-2.d0/3.d0*sw/cw*e,
     &                  (1.d0-4.d0/3.d0*csw2)/(2.d0*sw*cw)*e/)
      zdd=(/sw/cw/3.d0*e,-(1.d0-2.d0/3.d0*csw2)/(2.d0*sw*cw)*e/)

c W boson coupling to fermions:

      wff=(/ocom,e/sqrt(2.d0)/sw/)

c CKM mixing (real), if included:

      wud=wff*V_ud
      wus=wff*V_us
      wub=wff*V_ub
      wcd=wff*V_cd
      wcs=wff*V_cs
      wcb=wff*V_cb
      wtd=wff*V_td
      wts=wff*V_ts

c The Higgs boson coupling to fermions:

      hff=-e/(2.d0*sw*cmw)

      hee=sqrt(cmplx(me**2,-me*game,dp))*hff
      hmm=sqrt(cmplx(mmu**2,-mmu*gammu,dp))*hff
      hll=sqrt(cmplx(mtau**2,-mtau*gamtau,dp))*hff
      huu=sqrt(cmplx(mu**2,-mu*gamu,dp))*hff
      hdd=sqrt(cmplx(md**2,-md*gamd,dp))*hff
      hcc=sqrt(cmplx(mc**2,-mc*gamc,dp))*hff
      hss=sqrt(cmplx(ms**2,-ms*gams,dp))*hff
      htt=sqrt(cmplx(mt**2,-mt*gamt,dp))*hff
      if (ihtt == 1) then
       htt=htt*(/cmplx(httv,htta,dp),cmplx(httv,-htta,dp)/)
      end if
      hbb=sqrt(cmplx(mb**2,-mb*gamb,dp))*hff

c Gluon coupling to quarks:

      gqq=(/cmplx(es,0.d0,dp),cmplx(es,0.d0,dp)/)

c Triple gauge boson couplings:

      aww=cmplx(e,0.d0,dp)
      zww=-cw/sw*e
      ggg=-gqq(1)

c Quartic gauge boson couplings:

      aaww=cmplx(e**2,0.d0,dp)
      azww=-cw/sw*e**2
      zzww=ccw2/csw2*e**2
      wwww=-1.d0/csw2*e**2
      gggg=gqq(1)**2

c Higgs boson couplings to Z and W bosons:

      hzz=e*cmw/sw/ccw2
      hww=e*cmw/sw

c The triple Higgs boson coupling

      hhh=-3.d0*cmh**2/(2.d0*sw*cmw)*e

c Quartic Higgs boson couplings

      hhww=e**2/(2.d0*csw2)
      hhzz=e**2/(2.d0*csw2*ccw2)
      hhhh=-3.d0*cmh**2/(4.d0*csw2*cmw**2)*e**2

c Scalar-fermion couplings:

c      csff=e/(sqrt(2.d0)*sw*cmw)
c      c3ff=cmplx(0.d0,-e,dp)/(2.d0*sw*cmw)

c Scalar-gauge boson couplings:

c      csaw=e*cmw
c      cszw=-e*cmw*sw/cw 

c Outgoing phi-(p1), incoming phi-(p2):

c      cass=-e
c      czss=e*(sw**2-cw**2)/(2.d0*sw*cw)

c Outgoing phi3(p1), incoming phi-(p2):

c      cw3s=cmplx(0.d0,e,dp)/(2.d0*sw)

c Outgoing H(p1), incoming phi-(p2):

c      cwhs=-e/(2.d0*sw)

c Outgoing phi3(p1), incoming H(p2):

c      cz3h=cmplx(0.d0,e,dp)/(2.d0*sw*cw)

c Triple scalar couplings:

c      chss=-e*cmh**2/(2.d0*sw*cmw)


c----------------------------------------------------------------------c
c HLS couplings
c----------------------------------------------------------------------c

c Photon-rho0 mixing:

      if (iarho == 0) then
       arho=(0.d0,0.d0)
      else
       arho=cmplx(0.d0,e*f_arho,dp)
      end if

c Photon-omega mixing:

      if (iaome == 0) then
       aome=(0.d0,0.d0)
      else
       aome=cmplx(0.d0,e*f_aome,dp)
      end if

c Photon-phi mixing:

      if (iaphi == 0) then
       aphi=(0.d0,0.d0)
      else
       aphi=cmplx(0.d0,e*f_aphi,dp)
      end if

c Photon-rho1 mixing:

      if (iarho1 == 0) then
       arh1=(0.d0,0.d0)
      else
       arh1=cmplx(0.d0,e,dp)
      end if

c Photon-rho2 mixing:

      if (iarho2 == 0) then
       arh2=(0.d0,0.d0)
      else
       arh2=cmplx(0.d0,e,dp)
      end if

c Triple photon-pion-pion coupling of scalar QED:

      app=cmplx(e,0.d0,dp)
      iapp=1  ! running
      iapp=0

c Quartic photon-pion coupling of scalar QED:

      aapp=cmplx(2.d0*e**2,0.d0,dp)
c      aapp=(0.d0,0.d0)
      iaapp=2   ! running
      iaapp1=3  ! running, if one photon in the coupling is on shell
      iaapp=0
      iaapp1=0

c Triple photon-K+K- coupling of scalar QED:

      akk=cmplx(e,0.d0,dp)
      iakk=0  ! fixed

c Triple photon-K0K0 coupling of scalar QED:

      ak0k0=cmplx(e,0.d0,dp)
      iak0k0=0  ! fixed

c Triple rho0-pion-pion coupling, as in scalar QED:

      r0pp=cmplx(g_rhopipi,0.d0,dp)
      ir0pp=0  ! fixed

c Triple rho0-K+K- coupling, as in scalar QED:

      r0kk=cmplx(e,0.d0,dp)
      ir0kk=0  ! fixed

c Triple rho0-K0K0 coupling, as in scalar QED:

      r0k0k0=cmplx(e,0.d0,dp)
      ir0k0k0=0  ! fixed

c Triple rho1-pion-pion coupling, as in scalar QED:

      r1pp=cmplx(e,0.d0,dp)
      ir1pp=0  ! fixed

c Triple rho2-pion-pion coupling, as in scalar QED:

      r2pp=cmplx(e,0.d0,dp)
      ir1pp=0  ! fixed

c Triple omega-pion-pion coupling, as in scalar QED:

      opp=cmplx(g_omepipi,0.d0,dp)
      iopp=0  ! fixed

c Triple omega-K+K- coupling, as in scalar QED:

      okk=cmplx(e,0.d0,dp)
      iokk=0  ! fixed

c Triple omega-K0K0 coupling, as in scalar QED:

      ok0k0=cmplx(e,0.d0,dp)
      iok0k0=0  ! fixed

c Triple phi-pion-pion coupling, as in scalar QED:

      phpp=cmplx(g_phipipi,0.d0,dp)
      iphpp=0  ! fixed

c Triple phi-K+K- coupling, as in scalar QED:

      phkk=cmplx(g_phikk,0.d0,dp)
      iphkk=0  ! fixed

c Triple phi-K0K0 coupling, as in scalar QED:

      phk0k0=cmplx(g_phikk,0.d0,dp)
      iphk0k0=0  ! fixed

c Triple pion-photon-photon coupling:

c      paa=cmplx(e**2*g_piaa,0.d0,dp)
      paa=cmplx(0.d0,e**2*g_piaa,dp)
      ipaa=0  ! fixed

c Triple pi0-photon-rho0 coupling:

      par0=cmplx(e*g_rhoapi,0.d0,dp)
      ipar0=0  ! fixed

c Triple pi- photon rho+ coupling:

      parp=cmplx(e*g_rhoapi,0.d0,dp)
c      parp=(0.d0,0.d0)
      iparp=0  ! fixed

c Triple pi+ photon rho- coupling:

      parm=cmplx(e*g_rhoapi,0.d0,dp)
c      parm=(0.d0,0.d0)
      iparm=0  ! fixed

c Triple pi0-photon-omega coupling:

      pao=cmplx(e*g_omeapi,0.d0,dp)
      ipao=0  ! fixed

c Triple pi0-omega-rho0 coupling:

      por0=cmplx(0.d0,g_omerhopi,dp)
      ipor0=0  ! fixed

c Triple pi- omega rho+ coupling:

      porp=cmplx(0.d0,g_omerhopi,dp)
      iporp=0  ! fixed

c Triple pi+ omega rho- coupling:

      porm=cmplx(0.d0,g_omerhopi,dp)
      iporm=0  ! fixed

c Quartic photon-rh0-pion-pion coupling, same as of scalar QED:

      arpp=cmplx(2.d0*e*g_rhopipi,0.d0,dp)
c      arpp=cmplx(0.d0,0.d0,dp)
      iarpp=0  ! fixed

c Quartic rho- rh0+ pi- pi+ coupling, same as of scalar QED:

      rrpp=cmplx(2.d0*g_rhopipi**2,0.d0,dp)
      irrpp=0  ! fixed

c Quartic photon pi0 pi+ pi- coupling:

      appp=cmplx(0.d0,-e*g_apipipi,dp)
c      appp=cmplx(0.d0,0.d0,dp)
      iappp=0  ! fixed

c Quartic omega pi0 pi+ pi- coupling:

      oppp=cmplx(0.d0,-g_omepipipi,dp)
c      oppp=cmplx(0.d0,0.d0,dp)
      ioppp=0  ! fixed

      end


